import pandas as pd


def calculate_single_load_total_weight(single_load):
    return single_load["unit_weight"] * single_load["unit_number"]


def calculate_single_load_total_volume(single_load):
    return (
        single_load["unit_length"]
        * single_load["unit_width"]
        * single_load["unit_height"]
        * single_load["unit_number"]
        / 1000000
    )


def calculate_single_load_total_quantities(single_load: pd.Series) -> pd.Series:
    single_load["total_weight_in_kg"] = (
        single_load["total_weight_in_kg"]
        if not pd.isna(single_load["total_weight_in_kg"]) and single_load["total_weight_in_kg"] > 0
        else calculate_single_load_total_weight(single_load)
    )
    single_load["total_volume_in_cbm"] = (
        single_load["total_volume_in_cbcm"] / 1000000
        if not pd.isna(single_load["total_volume_in_cbcm"]) and single_load["total_volume_in_cbcm"] > 0
        else calculate_single_load_total_volume(single_load)
    )
    single_load["total_number"] = single_load["unit_number"]
    return single_load


def calculate_taxable_weight(total_weight: float, total_volume: float):
    return round(max(total_volume / 6, total_weight / 1000) * 1000, 3)


def calculate_weight_measurable(total_weight: float, total_volume: float):
    return round(max(total_weight / 1000, total_volume, 1), 3)



import pandas as pd
import numpy as np
from pandas.util.testing import assert_frame_equal

from ovretl.shipments_utils.add_client_quotation_shipment_index import add_client_quotation_shipment_index


def test_add_client_quotation_index():
    quotations_df = pd.DataFrame(
        data={
            "client_name": ["Foo", "Bar", "Foo", "Bar", "Foo"],
            "created_at": [3, 4, 2, 7, 10],
            "foresea_name": ["BATE", "BATZ", "BOUM", "BLOB", "BOPS"],
            "shipment_status": ["purchase_ready", "booking_request", "booked", "new", "not_answered"],
        }
    )
    result = add_client_quotation_shipment_index(quotations_df)
    result_should_be = pd.DataFrame(
        data={
            "client_name": ["Bar", "Bar", "Foo", "Foo", "Foo"],
            "quotation_number": [1, 2, 1, 2, 3],
            "created_at": [4, 7, 2, 3, 10],
            "foresea_name": ["BATZ", "BLOB", "BOUM", "BATE", "BOPS"],
            "shipment_status": ["booking_request", "new", "booked", "purchase_ready", "not_answered"],
            "shipment_number": [1, np.nan, 1, np.nan, np.nan],
        }
    )
    assert_frame_equal(result, result_should_be)



import numpy as np
import pandas as pd
from ovretl import add_billing_info
from pandas.util.testing import assert_frame_equal


def test_add_billing_info():
    billings_df = pd.DataFrame(
        data={
            "shipment_id": ["0", "0", "1", "1", "2", "2", "4"],
            "billing_number": ["ABC", "DEF", "GHI", np.nan, np.nan, np.nan, "JKL"],
            "billing_address_country": ["fr", "cn", "uk", np.nan, np.nan, np.nan, "fr"],
            "status": ["in_modification", "in_modification", "due", "in_modification", "paid", "in_modification",
                       "paid"],
        }
    )
    shipments_df = pd.DataFrame(
        data={"shipment_id": ["0", "1", "2", "3"], "billing_address_country": ["us", np.nan, np.nan, "be"]}
    )
    result = add_billing_info(shipments_df=shipments_df, billings_df=billings_df)
    result_should_be = pd.DataFrame(
        data={
            "shipment_id": ["0", "1", "2", "3"],
            "billing_numbers": ["ABC, DEF", "", "", ""],
            "billing_status": ["awaiting_invoice", "due", "paid", "awaiting_invoice"],
            "billing_address_country": ["fr", "uk", np.nan, "be"],
        }
    )
    assert_frame_equal(result, result_should_be)



import psycopg2

from ovretl.db_utils.fetch_db_credentials import fetch_db_credentials


def execute_simple_query(query: str, dbname: str, stage="dev", select=False):
    result = None
    credentials = fetch_db_credentials(stage=stage)
    conn = psycopg2.connect(
        host=credentials["host"],
        user=credentials["username"],
        password=credentials["password"],
        dbname=dbname,
        port=6543,
    )
    cur = conn.cursor()
    cur.execute(query)
    return cur.description
    if select:
        result = cur.fetchall()
    cur.close()
    conn.commit()
    conn.close()
    return result



from ovretl.billings_utils.add_billing_info import *
from ovretl.kronos_propositions_utils.add_missing_categories import *
from ovretl.kronos_propositions_utils.match_kronos_propositions_and_shipments import *
from ovretl.containers_utils.calculate_entity_containers import *
from ovretl.containers_utils.calculate_single_container_teus import *
from ovretl.containers_utils.merge_shipments_with_containers import *
from ovretl.employees_utils.find_shipment_employee_name import *
from ovretl.employees_utils.join_employees_associations_to_employees_name import *
from ovretl.loads_utils.calculate_entity_loads import *
from ovretl.loads_utils.calculate_single_load_total_quantities import *
from ovretl.loads_utils.merge_shipments_with_loads import *
from ovretl.performances_utils.calculate_business_hours_delta import *
from ovretl.performances_utils.calculate_business_hours_delta import *
from ovretl.performances_utils.calculate_shipments_treatment_steps_time import *
from ovretl.performances_utils.calculate_number_purchase_readied import *
from ovretl.prices_utils.add_category_prices_to_shipments import *
from ovretl.prices_utils.add_prices_to_shipments import *
from ovretl.shipments_utils.determine_import_status import *
from ovretl.shipments_utils.add_client_quotation_shipment_index import *
from ovretl.shipowners_utils.find_shipment_category_shipowner import *
from ovretl.tracking_utils.extract_event_date import *
from ovretl.tracking_utils.join_tracking_events import *
from ovretl.shipment_orchestration_utils import *
from ovretl.db_utils.select_from_table import select_from_table



import pandas as pd


def join_billings_numbers(billings_numbers):
    return ", ".join(billings_numbers) if billings_numbers.apply(lambda x: not pd.isna(x)).all() else ""


def compute_billing_status(statuses: pd.Series):
    if len(statuses) == 0:
        return "awaiting_invoice"
    if statuses.isin(["in_modification"]).all():
        return "awaiting_invoice"
    if statuses.isin(["due"]).any():
        return "due"
    if statuses.isin(["available"]).any():
        return "available"

    return "paid"


def add_billing_info(shipments_df: pd.DataFrame, billings_df: pd.DataFrame):
    billings_df.loc[:, "billing_numbers"] = billings_df.groupby("shipment_id")["billing_number"].transform(
        join_billings_numbers
    )
    billings_df.loc[:, "billing_status"] = billings_df.groupby("shipment_id")["status"].transform(
        compute_billing_status
    )
    billings_df = billings_df.drop_duplicates(subset=["shipment_id"])
    shipments_df = pd.merge(
        shipments_df,
        billings_df[["shipment_id", "billing_numbers", "billing_status", "billing_address_country"]],
        on="shipment_id",
        how="left",
    )
    shipments_df.loc[:, "billing_address_country"] = shipments_df.apply(
        lambda row: row["billing_address_country_y"]
        if pd.notna(row["billing_address_country_y"])
        else row["billing_address_country_x"],
        axis=1,
    )
    shipments_df = shipments_df.drop(columns=["billing_address_country_x", "billing_address_country_y"])
    shipments_df.loc[:, "billing_numbers"] = shipments_df["billing_numbers"].fillna("")
    shipments_df.loc[:, "billing_status"] = shipments_df["billing_status"].fillna("awaiting_invoice")
    return shipments_df
